with Ada.Exceptions;
with System;
with Ada.Unchecked_Conversion;
with Ada.Text_IO;

package body Mcc.Parallel is
   Was_Initialized : Boolean := False;
   type Read_Write is (Read,Write);
   Control_Pin_Status : Read_Write;
   Port_Name : Character;

   pragma Linker_Options("-lkernel32");

   function GetLastError return Integer;
   pragma Import(Stdcall, GetLastError, "GetLastError");

   function CreateFile(
      lpFileName             : System.Address;
      dwDesiredAccess        : Integer;
      dwShareMode            : Integer;
      lpSecurityAttributes   : System.Address;
      dwCreationDistribution : Integer;
      dwFlagsAndAttributes   : Integer;
      hTemplateFile          : System.Address
   ) return System.Address;
   pragma Import(Stdcall,CreateFile,"CreateFileA");

   procedure DeleteFile(
      lpFileName : System.Address);
   pragma Import(Stdcall,DeleteFile,"DeleteFileA");

   function DeviceIoControl(
      hDevice          : System.Address;
      dwIoControlCode  : Integer;
      lpInBuffer       : System.Address;
      nInBufferSize    : Integer;
      lpOutBuffer      : System.Address;
      nOutBufferSize   : Integer;
      lpBytesReturned  : System.Address;
      lpOverlapped     : System.Address
   ) return Integer;
   pragma Import(Stdcall, DeviceIoControl,"DeviceIoControl");

   Portio_Device : System.Address;

   --------------------------
   -- Unload_Parallel_Port --
   --------------------------

   procedure Unload_Parallel_Port is
      Delete_Name : constant String := "\\.\PORTIO95" & Character'First;
   begin
      DeleteFile(Delete_Name'Address);
      Was_Initialized := False;
   end Unload_Parallel_Port;

   --------------------
   -- Is_Control_Pin --
   --------------------

   function Is_Control_Pin(Pin : in Readable_Pin) return Boolean is
   begin
      case Pin is
         when PIN1|PIN14|PIN16|PIN17 =>
            return true;
         when others =>
            return false;
      end case;
   end Is_Control_Pin;

   function Is_Control_Pin(Pin : in Writeable_Pin) return Boolean is
   begin
      case Pin is
         when PIN1|PIN14|PIN16|PIN17 =>
            return true;
         when others =>
            return false;
      end case;
   end Is_Control_Pin;

   ------------------------------
   -- Initialize_Parallel_Port --
   ------------------------------

   procedure Initialize_Parallel_Port
     (Port_Number : in Parallel_Port_Number)
   is
      Vxd_Name  : constant String := "\\.\PORTIO95.VXD" & Character'First;

      Create_New : constant := 1;
      Portio_Setport            : constant := 16#1008#;
      File_Flag_Delete_On_Close : constant := 16#4000000#;
      Error_Code : Integer;
      function Convert is new Ada.Unchecked_Conversion(
         System.Address,Integer);
   begin
      if Was_Initialized then
         Unload_Parallel_Port;
      end if;

      Port_Name := Character'Val(Character'Pos('0') + Port_Number);

      Portio_Device := CreateFile(Vxd_Name'Address,
         0,
         0,
         System.Null_Address,
         Create_New,
         File_Flag_Delete_On_Close,
         System.Null_Address);

      declare
         Portio_Integer : Integer := Convert(Portio_Device);
      begin
         if Portio_Integer = 0 or Portio_Integer = -1 then
            Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
               "Open of VXD failed-- error code:" &
               Integer'Image(GetLastError));
         end if;
      end;

      Error_Code := DeviceIoControl(Portio_Device,
         Portio_Setport,
         Port_Name'Address,
         Port_Name'Size/8,
--         16#1007#,
--         System.Null_Address,
--         0,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Text_IO.Put("Port name:" & Port_Name & " error:");
         Ada.Text_IO.Put_Line(Integer'Image(Error_Code));
         Ada.Text_IO.Put_Line("size:" & Integer'Image(Port_Name'Size/8));
         Ada.Text_IO.Put_Line(Integer'Image(Convert(Port_Name'Address)));
         Ada.Text_IO.Put_Line("press enter to terminate");
         Ada.Text_IO.Skip_Line;
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Setport failed-- error code:" &
            Integer'Image(GetLastError));
      end if;

      Was_Initialized := True; -- need to do this before Make_Control_Pins
      Make_Control_Pins_Write;
   end Initialize_Parallel_Port;

   ----------------------------
   -- Make_Control_Pins_Read --
   ----------------------------

   procedure Make_Control_Pins_Read is
      Error_Code : Integer;
      Portio_Set_Pins_1_14_16_17_AS_INPUT  : constant := 16#1018#;
   begin
      if not Was_Initialized then
         raise No_Port_Initialized;
      end if;

      Error_Code := DeviceIoControl(Portio_Device,
         Portio_Set_Pins_1_14_16_17_AS_INPUT,
         System.Null_Address,
         0,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Set control to input failed");
      end if;

      Control_Pin_Status := Read;
   end Make_Control_Pins_Read;

   -----------------------------
   -- Make_Control_Pins_Write --
   -----------------------------

   procedure Make_Control_Pins_Write is
      Error_Code : Integer;
      Portio_Set_Pins_1_14_16_17_AS_OUTPUT : constant := 16#1019#;
   begin
      if not Was_Initialized then
         raise No_Port_Initialized;
      end if;

      Error_Code := DeviceIoControl(Portio_Device,
         Portio_Set_Pins_1_14_16_17_AS_OUTPUT,
         System.Null_Address,
         0,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Set control to output failed");
      end if;

      Control_Pin_Status := Write;
   end Make_Control_Pins_Write;

   -------------------------
   -- Read_Control_Nibble --
   -------------------------

   procedure Read_Control_Nibble
     (Value : out Nibble)
   is
      Error_Code : Integer;
      Returned_Value : Character;
      Portio_Readcreg: constant := 16#1009#;
   begin
      if Control_Pin_Status = Write then
         raise Control_Write_Only;
      end if;

      Error_Code := DeviceIoControl(Portio_Device,
         Portio_Readcreg,
         System.Null_Address,
         0,
         Returned_Value'Address,
         Returned_Value'Size/8,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Read control nibble failed");
      end if;

      Value := Nibble(Character'Pos(Returned_Value));
   end Read_Control_Nibble;

   ----------------------
   -- Read_Data_Nibble --
   ----------------------

   procedure Read_Data_Nibble
     (Value : out Nibble)
   is
      Error_Code     : Integer;
      Returned_Value : Character;
      Portio_Readsreg: constant := 16#100E#;
   begin
      Error_Code := DeviceIoControl(Portio_Device,
         Portio_Readsreg,
         System.Null_Address,
         0,
         Returned_Value'Address,
         Returned_Value'Size/8,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Read data nibble failed");
      end if;

      Value := Nibble(Character'Pos(Returned_Value));
   end Read_Data_Nibble;

   --------------
   -- Read_Pin --
   --------------

   procedure Read_Pin
     (Pin_Number : in Readable_Pin;
      Value      : out Boolean)
   is
      Read_Code : constant array(Readable_Pin) of Integer :=
         (PIN1  => 16#102B#,
          PIN11 => 16#101F#,
          PIN12 => 16#101C#,
          PIN13 => 16#101B#,
          PIN14 => 16#102D#,
          PIN15 => 16#101A#,
          PIN16 => 16#102E#,
          PIN17 => 16#1030#);
      Error_Code     : Integer;
      Returned_Value : Character;
   begin
      if Is_Control_Pin(Pin_Number) and then
         Control_Pin_Status = Write
      then
         raise Control_Write_Only;
      end if;

      Error_Code := DeviceIOControl(Portio_Device,
         Read_Code(Pin_Number),
         System.Null_Address,
         0,
         Returned_Value'Address,
         Returned_Value'Size/8,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Read pin failed on pin: " & Readable_Pin'Image(Pin_Number));
      end if;

      Value := (Returned_Value /= Character'First);
   end Read_Pin;

   -------------
   -- Set_Pin --
   -------------

   procedure Set_Pin
     (Pin_Number : in Writeable_Pin;
      Value      : in Boolean)
   is
      Write_Code : constant array(Boolean'Range,
         Writeable_Pin'Range) of Integer :=
         (False =>
            (PIN1  => 16#1040#,
             PIN2  => 16#1020#,
             PIN3  => 16#1021#,
             PIN4  => 16#1022#,
             PIN5  => 16#1023#,
             PIN6  => 16#1024#,
             PIN7  => 16#1025#,
             PIN8  => 16#1026#,
             PIN9  => 16#1027#,
             PIN14 => 16#1041#,
             PIN16 => 16#1042#,
             PIN17 => 16#1043#),
          True =>
            (PIN1  => 16#1044#,
             PIN2  => 16#1010#,
             PIN3  => 16#1011#,
             PIN4  => 16#1012#,
             PIN5  => 16#1013#,
             PIN6  => 16#1014#,
             PIN7  => 16#1015#,
             PIN8  => 16#1016#,
             PIN9  => 16#1017#,
             PIN14 => 16#1045#,
             PIN16 => 16#1046#,
             PIN17 => 16#1047#));
      Error_Code     : Integer;
   begin
      if Is_Control_Pin(Pin_Number) and then
         Control_Pin_Status = Read
      then
         raise Control_Read_Only;
      end if;

      Ada.Text_Io.Put_Line("setting a pin");
      Error_Code := DeviceIOControl(Portio_Device,
         Write_Code(Value,Pin_Number),
         System.Null_Address,
         0,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Write pin failed on pin: " & Writeable_Pin'Image(Pin_Number));
      end if;

   end Set_Pin;

   --------------------------
   -- Write_Control_Nibble --
   --------------------------

   procedure Write_Control_Nibble
     (Value : in Nibble)
   is
      Error_Code   : Integer;
      Output_Value : Character := Character'Val(Value);
      Portio_Send_Byte_To_Pins1_14_16_17 : constant := 16#100B#;
   begin
      if Control_Pin_Status = Read then
         raise Control_Read_Only;
      end if;

      Error_Code := DeviceIOControl(Portio_Device,
         Portio_Send_Byte_To_Pins1_14_16_17,
         Output_Value'Address,
         Output_Value'Size/8,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);

      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Write control nibble failed");
      end if;
   end Write_Control_Nibble;

   ---------------------
   -- Write_Data_Byte --
   ---------------------

   procedure Write_Data_Byte
     (Value : in Byte)
   is
      Error_Code   : Integer;
      Output_Value : Character := Character'Val(Value);
      Portio_Write_Dreg : constant := 16#100C#;
   begin
      Error_Code := DeviceIOControl(Portio_Device,
         Portio_Write_Dreg,
         Output_Value'Address,
         Output_Value'Size/8,
         System.Null_Address,
         0,
         System.Null_Address,
         System.Null_Address);
      if Error_Code = 0 then
         Ada.Exceptions.Raise_Exception(Portio_exception'Identity,
            "Write data byte failed");
      end if;
   end Write_Data_Byte;

end Mcc.Parallel;